@@ -187,9 +187,6 @@ const AllView = new Lang.Class({
     _init: function() {
         this.parent();
 
-        this._grid.actor.y_align = Clutter.ActorAlign.START;
-        this._grid.actor.y_expand = true;
-
         let box = new St.BoxLayout({ vertical: true });
         this._stack = new St.Widget({ layout_manager: new AllViewLayout() });
         this._stack.add_actor(this._grid.actor);
@@ -279,12 +276,8 @@ const AllView = new Lang.Class({
                 this._eventBlocker.reactive = isOpen;
                 this._currentPopup = isOpen ? popup : null;
                 this._updateIconOpacities(isOpen);
-                if (isOpen) {
+                if (isOpen)
                     this._ensureIconVisible(popup.actor);
-                    this._grid.actor.y = popup.parentOffset;
-                } else {
-                    this._grid.actor.y = 0;
-                }
             }));
     },
 
@@ -323,6 +316,8 @@ const FrequentView = new Lang.Class({
     loadApps: function() {
         let mostUsed = this._usage.get_most_used ("");
         for (let i = 0; i < mostUsed.length; i++) {
+            if (!mostUsed[i].get_app_info().should_show())
+                continue;
             let appIcon = new AppIcon(mostUsed[i]);
             this._grid.addItem(appIcon.actor, -1);
         }
@@ -334,42 +329,6 @@ const Views = {
     ALL: 1
 };
 
-const ControlsBoxLayout = Lang.Class({
-    Name: 'ControlsBoxLayout',
-    Extends: Clutter.BoxLayout,
-
-    /**
-     * Override the BoxLayout behavior to use the maximum preferred width of all
-     * buttons for each child
-     */
-    vfunc_get_preferred_width: function(container, forHeight) {
-        let maxMinWidth = 0;
-        let maxNaturalWidth = 0;
-        for (let child = container.get_first_child();
-             child;
-             child = child.get_next_sibling()) {
-             let [minWidth, natWidth] = child.get_preferred_width(forHeight);
-             maxMinWidth = Math.max(maxMinWidth, minWidth);
-             maxNaturalWidth = Math.max(maxNaturalWidth, natWidth);
-        }
-        let childrenCount = container.get_n_children();
-        let totalSpacing = this.spacing * (childrenCount - 1);
-        return [maxMinWidth * childrenCount + totalSpacing,
-                maxNaturalWidth * childrenCount + totalSpacing];
-    },
-
-    vfunc_set_container: function(container) {
-        if(this._styleChangedId) {
-            this._container.disconnect(this._styleChangedId);
-            this._styleChangedId = 0;
-        }
-        if(container != null)
-            this._styleChangedId = container.connect('style-changed', Lang.bind(this,
-                    function() { this.spacing = this._container.get_theme_node().get_length('spacing'); }));
-        this._container = container;
-    }
-});
-
 const AppDisplay = new Lang.Class({
     Name: 'AppDisplay',
 
@@ -410,9 +369,9 @@ const AppDisplay = new Lang.Class({
                                           x_expand: true, y_expand: true });
         this.actor.add(this._viewStack, { expand: true });
 
-        let layout = new ControlsBoxLayout({ homogeneous: true });
-        this._controls = new St.Widget({ style_class: 'app-view-controls' });
-        this._controls.set_layout_manager(layout);
+        let layout = new Clutter.BoxLayout({ homogeneous: true });
+        this._controls = new St.Widget({ style_class: 'app-view-controls',
+                                         layout_manager: layout });
         this.actor.add(new St.Bin({ child: this._controls }));
 
 
@@ -615,11 +574,7 @@ const FolderIcon = new Lang.Class({
         // Position the popup above or below the source icon
         if (side == St.Side.BOTTOM) {
             this._popup.actor.show();
-            let closeButtonOffset = -this._popup.closeButton.translation_y;
-            let y = this.actor.y - this._popup.actor.height;
-            let yWithButton = y - closeButtonOffset;
-            this._popup.parentOffset = yWithButton < 0 ? -yWithButton : 0;
-            this._popup.actor.y = Math.max(y, closeButtonOffset);
+            this._popup.actor.y = this.actor.y - this._popup.actor.height;
             this._popup.actor.hide();
         } else {
             this._popup.actor.y = this.actor.y + this.actor.height;
@@ -642,7 +597,6 @@ const AppFolderPopup = new Lang.Class({
         this._arrowSide = side;
 
         this._isOpen = false;
-        this.parentOffset = 0;
 
         this.actor = new St.Widget({ layout_manager: new Clutter.BinLayout(),
                                      visible: false,
@@ -666,31 +620,17 @@ const AppFolderPopup = new Lang.Class({
         this.actor.add_actor(this._boxPointer.actor);
         this._boxPointer.bin.set_child(this._view.actor);
 
-        this.closeButton = Util.makeCloseButton();
-        this.closeButton.connect('clicked', Lang.bind(this, this.popdown));
-        this.actor.add_actor(this.closeButton);
+        let closeButton = Util.makeCloseButton();
+        closeButton.connect('clicked', Lang.bind(this, this.popdown));
+        this.actor.add_actor(closeButton);
 
-        this._boxPointer.actor.bind_property('opacity', this.closeButton, 'opacity',
+        this._boxPointer.actor.bind_property('opacity', closeButton, 'opacity',
                                              GObject.BindingFlags.SYNC_CREATE);
 
-        global.focus_manager.add_group(this.actor);
-
         source.actor.connect('destroy', Lang.bind(this,
             function() {
                 this.actor.destroy();
             }));
-        this.actor.connect('key-press-event', Lang.bind(this, this._onKeyPress));
-    },
-
-    _onKeyPress: function(actor, event) {
-        if (!this._isOpen)
-            return false;
-
-        if (event.get_key_symbol() != Clutter.KEY_Escape)
-            return false;
-
-        this.popdown();
-        return true;
     },
 
     toggle: function() {
@@ -705,7 +645,6 @@ const AppFolderPopup = new Lang.Class({
             return;
 
         this.actor.show();
-        this.actor.navigate_focus(null, Gtk.DirectionType.TAB_FORWARD, false);
 
         this._boxPointer.setArrowActor(this._source.actor);
         this._boxPointer.show(BoxPointer.PopupAnimation.FADE |
