@@ -126,6 +126,11 @@ const BackgroundCache = new Lang.Class({
     },
 
     removeImageContent: function(content) {
+        let filename = content.get_filename();
+
+        if (filename && this._fileMonitors[filename])
+            delete this._fileMonitors[filename];
+
         this._removeContent(this._images, content);
     },
 
@@ -426,7 +431,7 @@ const Background = new Lang.Class({
         content.brightness = this._brightness;
         content.vignette_sharpness = this._vignetteSharpness;
 
-        this._cache.removeImageContent(content);
+        this._cache.removeImageContent(this._images[index].content);
         this._images[index].content = content;
         this._watchCacheFile(filename);
     },
@@ -742,11 +747,15 @@ const BackgroundManager = new Lang.Class({
                                    time: FADE_ANIMATION_TIME,
                                    transition: 'easeOutQuad',
                                    onComplete: Lang.bind(this, function() {
-                                       if (this.background == background) {
+                                       if (this._newBackground == newBackground) {
                                            this.background = newBackground;
                                            this._newBackground = null;
-                                           background.actor.destroy();
+                                       } else {
+                                           newBackground.actor.destroy();
                                        }
+
+                                       background.actor.destroy();
+
                                        this.emit('changed');
                                    })
                                  });
