@@ -100,6 +100,7 @@ const DateMenuButton = new Lang.Class({
 
         Shell.AppSystem.get_default().connect('installed-changed',
                                               Lang.bind(this, this._appInstalledChanged));
+        this._appInstalledChanged();
 
         item = this.menu.addSettingsAction(_("Date & Time Settings"), 'gnome-datetime-panel.desktop');
         if (item) {
@@ -156,16 +157,14 @@ const DateMenuButton = new Lang.Class({
     },
 
     _appInstalledChanged: function() {
-        this._calendarApp = undefined;
-        this._updateEventsVisibility();
+        let app = Shell.AppSystem.get_default().lookup_app('gnome-clocks.desktop');
+        this._openClocksItem.actor.visible = app !== null;
     },
 
     _updateEventsVisibility: function() {
         let visible = this._eventSource.hasCalendars;
-        this._openCalendarItem.actor.visible = visible &&
-            (this._getCalendarApp() != null);
-        this._openClocksItem.actor.visible = visible &&
-            (this._getClockApp() != null);
+        this._openCalendarItem.actor.visible = visible;
+        this._openClocksItem.actor.visible = visible;
         this._separator.visible = visible;
         if (visible) {
           let alignment = 0.25;
@@ -218,26 +217,10 @@ const DateMenuButton = new Lang.Class({
         this._date.set_text(displayDate.toLocaleFormat(dateFormat));
     },
 
-    _getCalendarApp: function() {
-        if (this._calendarApp !== undefined)
-            return this._calendarApp;
-
-        let apps = Gio.AppInfo.get_recommended_for_type('text/calendar');
-        if (apps && (apps.length > 0))
-            this._calendarApp = apps[0];
-        else
-            this._calendarApp = null;
-        return this._calendarApp;
-    },
-
-    _getClockApp: function() {
-        return Shell.AppSystem.get_default().lookup_app('gnome-clocks.desktop');
-    },
-
     _onOpenCalendarActivate: function() {
         this.menu.close();
 
-        let app = this._getCalendarApp();
+        let app = Gio.AppInfo.get_default_for_type('text/calendar', false);
         if (app.get_id() == 'evolution.desktop')
             app = Gio.DesktopAppInfo.new('evolution-calendar.desktop');
         app.launch([], global.create_app_launch_context());
@@ -245,7 +228,7 @@ const DateMenuButton = new Lang.Class({
 
     _onOpenClocksActivate: function() {
         this.menu.close();
-        let app = this._getClockApp();
+        let app = Shell.AppSystem.get_default().lookup_app('gnome-clocks.desktop');
         app.activate();
     }
 });
