@@ -118,6 +118,22 @@ const MonitorConstraint = new Lang.Class({
     }
 });
 
+const Monitor = new Lang.Class({
+    Name: 'Monitor',
+
+    _init: function(index, geometry) {
+        this.index = index;
+        this.x = geometry.x;
+        this.y = geometry.y;
+        this.width = geometry.width;
+        this.height = geometry.height;
+    },
+
+    get inFullscreen() {
+        return global.screen.get_monitor_in_fullscreen(this.index);
+    }
+})
+
 const defaultParams = {
     trackFullscreen: false,
     affectsStruts: false,
@@ -261,7 +277,7 @@ const LayoutManager = new Lang.Class({
         this.monitors = [];
         let nMonitors = screen.get_n_monitors();
         for (let i = 0; i < nMonitors; i++)
-            this.monitors.push(screen.get_monitor_geometry(i));
+            this.monitors.push(new Monitor(i, screen.get_monitor_geometry(i)));
 
         if (nMonitors == 1) {
             this.primaryIndex = this.bottomIndex = 0;
@@ -283,8 +299,10 @@ const LayoutManager = new Lang.Class({
 
     _updateHotCorners: function() {
         // destroy old hot corners
-        for (let i = 0; i < this.hotCorners.length; i++)
-            this.hotCorners[i].destroy();
+        this.hotCorners.forEach(function(corner) {
+            if (corner)
+                corner.destroy();
+        });
         this.hotCorners = [];
 
         let size = this.panelBox.height;
@@ -295,9 +313,9 @@ const LayoutManager = new Lang.Class({
             let cornerX = this._rtl ? monitor.x + monitor.width : monitor.x;
             let cornerY = monitor.y;
 
-            if (i != this.primaryIndex) {
-                let haveTopLeftCorner = true;
+            let haveTopLeftCorner = true;
 
+            if (i != this.primaryIndex) {
                 // Check if we have a top left (right for RTL) corner.
                 // I.e. if there is no monitor directly above or to the left(right)
                 let besideX = this._rtl ? monitor.x + 1 : cornerX - 1;
@@ -324,14 +342,15 @@ const LayoutManager = new Lang.Class({
                         break;
                     }
                 }
-
-                if (!haveTopLeftCorner)
-                    continue;
             }
 
-            let corner = new HotCorner(this, monitor, cornerX, cornerY);
-            corner.setBarrierSize(size);
-            this.hotCorners.push(corner);
+            if (haveTopLeftCorner) {
+                let corner = new HotCorner(this, monitor, cornerX, cornerY);
+                corner.setBarrierSize(size);
+                this.hotCorners.push(corner);
+            } else {
+                this.hotCorners.push(null);
+            }
         }
 
         this.emit('hot-corners-changed');
@@ -408,7 +427,8 @@ const LayoutManager = new Lang.Class({
 
         let size = this.panelBox.height;
         this.hotCorners.forEach(function(corner) {
-            corner.setBarrierSize(size);
+            if (corner)
+                corner.setBarrierSize(size);
         });
     },
 
@@ -728,8 +748,6 @@ const LayoutManager = new Lang.Class({
     // and shown otherwise)
     addChrome: function(actor, params) {
         this.uiGroup.add_actor(actor);
-        if (this.uiGroup.contains(global.top_window_group))
-            this.uiGroup.set_child_below_sibling(actor, global.top_window_group);
         this._trackActor(actor, params);
     },
 
@@ -902,13 +920,8 @@ const LayoutManager = new Lang.Class({
     },
 
     _updateFullscreen: function() {
-        for (let i = 0; i < this.monitors.length; i++)
-            this.monitors[i].inFullscreen = global.screen.get_monitor_in_fullscreen (i);
-
         this._updateVisibility();
         this._queueUpdateRegions();
-
-        this.emit('fullscreen-changed');
     },
 
     _windowsRestacked: function() {
