@@ -38,8 +38,12 @@ const Magnifier = imports.ui.magnifier;
 const XdndHandler = imports.ui.xdndHandler;
 const Util = imports.misc.util;
 
+const OVERRIDES_SCHEMA = 'org.gnome.shell.overrides';
 const DEFAULT_BACKGROUND_COLOR = Clutter.Color.from_pixel(0x2e3436ff);
 
+const A11Y_SCHEMA = 'org.gnome.desktop.a11y.keyboard';
+const STICKY_KEYS_ENABLE = 'stickykeys-enable';
+
 let componentManager = null;
 let panel = null;
 let overview = null;
@@ -67,7 +71,8 @@ let layoutManager = null;
 let _startDate;
 let _defaultCssStylesheet = null;
 let _cssStylesheet = null;
-let _workspacesSettings = null;
+let _overridesSettings = null;
+let _a11ySettings = null;
 
 function _sessionUpdated() {
     _loadDefaultStylesheet();
@@ -105,7 +110,6 @@ function start() {
 
 function _sessionsLoaded() {
     sessionMode.connect('updated', _sessionUpdated);
-    _initializePrefs();
     _initializeUI();
 
     shellDBusService = new ShellDBus.GnomeShell();
@@ -114,21 +118,6 @@ function _sessionsLoaded() {
     _sessionUpdated();
 }
 
-function _initializePrefs() {
-    let keys = new Gio.Settings({ schema: sessionMode.overridesSchema }).list_keys();
-    for (let i = 0; i < keys.length; i++)
-        Meta.prefs_override_preference_schema (keys[i], sessionMode.overridesSchema);
-
-    let workspacesSchema;
-    if (keys.indexOf('dynamic-workspaces') > -1)
-        workspacesSchema = sessionMode.overridesSchema;
-    else
-        workspacesSchema = 'org.gnome.mutter';
-
-     _workspacesSettings = new Gio.Settings({ schema: workspacesSchema });
-     _workspacesSettings.connect('changed::dynamic-workspaces', _queueCheckWorkspaces);
-}
-
 function _initializeUI() {
     // Ensure ShellWindowTracker and ShellAppUsage are initialized; this will
     // also initialize ShellAppSystem first.  ShellAppSystem
@@ -172,9 +161,14 @@ function _initializeUI() {
     layoutManager.init();
     overview.init();
 
+    _a11ySettings = new Gio.Settings({ schema: A11Y_SCHEMA });
+
     global.screen.override_workspace_layout(Meta.ScreenCorner.TOPLEFT,
                                             false, -1, 1);
-    global.display.connect('overlay-key', Lang.bind(overview, overview.toggle));
+    global.display.connect('overlay-key', Lang.bind(overview, function () {
+        if (!_a11ySettings.get_boolean (STICKY_KEYS_ENABLE))
+            overview.toggle();
+    }));
 
     // Provide the bus object for gnome-session to
     // initiate logouts.
@@ -194,6 +188,9 @@ function _initializeUI() {
         Scripting.runPerfScript(module, perfOutput);
     }
 
+    _overridesSettings = new Gio.Settings({ schema: OVERRIDES_SCHEMA });
+    _overridesSettings.connect('changed::dynamic-workspaces', _queueCheckWorkspaces);
+
     global.screen.connect('notify::n-workspaces', _nWorkspacesChanged);
 
     global.screen.connect('window-entered-monitor', _windowEnteredMonitor);
