@@ -6,10 +6,12 @@ const Meta = imports.gi.Meta;
 const St = imports.gi.St;
 const Shell = imports.gi.Shell;
 
+const Dash = imports.ui.dash;
 const Main = imports.ui.main;
 const Params = imports.misc.params;
 const Tweener = imports.ui.tweener;
 const ViewSelector = imports.ui.viewSelector;
+const WorkspaceThumbnail = imports.ui.workspaceThumbnail;
 
 const SIDE_CONTROLS_ANIMATION_TIME = 0.16;
 
@@ -269,6 +271,11 @@ const ThumbnailsSlider = new Lang.Class({
         return alwaysZoomOut;
     },
 
+    getNonExpandedWidth: function() {
+        let child = this.actor.get_first_child();
+        return child.get_theme_node().get_length('visible-width');
+    },
+
     getSlide: function() {
         if (!this.visible)
             return 0;
@@ -280,18 +287,16 @@ const ThumbnailsSlider = new Lang.Class({
         let child = this.actor.get_first_child();
         let preferredHeight = child.get_preferred_height(-1)[1];
         let expandedWidth = child.get_preferred_width(preferredHeight)[1];
-        let visibleWidth = child.get_theme_node().get_length('visible-width');
 
-        return visibleWidth / expandedWidth;
+        return this.getNonExpandedWidth() / expandedWidth;
     },
 
     getVisibleWidth: function() {
         let alwaysZoomOut = this._getAlwaysZoomOut();
         if (alwaysZoomOut)
             return this.parent();
-
-        let child = this.actor.get_first_child();
-        return child.get_theme_node().get_length('visible-width');
+        else
+            return this.getNonExpandedWidth();
     }
 });
 
@@ -309,6 +314,10 @@ const DashSlider = new Lang.Class({
         // available allocation
         this._dash.actor.x_expand = true;
         this._dash.actor.y_expand = true;
+
+        this.actor.x_align = Clutter.ActorAlign.START;
+        this.actor.y_expand = true;
+
         this.actor.add_actor(this._dash.actor);
 
         this._dash.connect('icon-size-changed', Lang.bind(this, this.updateSlide));
@@ -479,39 +488,77 @@ const MessagesIndicator = new Lang.Class({
 const ControlsManager = new Lang.Class({
     Name: 'ControlsManager',
 
-    _init: function(dash, thumbnails, viewSelector) {
-        this._dashSlider = new DashSlider(dash);
-        this.dashActor = this._dashSlider.actor;
-        this.dashSpacer = new DashSpacer();
-        this.dashSpacer.setDashActor(this.dashActor);
+    _init: function(searchEntry) {
+        this.dash = new Dash.Dash();
+        this._dashSlider = new DashSlider(this.dash);
+        this._dashSpacer = new DashSpacer();
+        this._dashSpacer.setDashActor(this._dashSlider.actor);
 
-        this._thumbnailsSlider = new ThumbnailsSlider(thumbnails);
-        this.thumbnailsActor = this._thumbnailsSlider.actor;
+        this._thumbnailsBox = new WorkspaceThumbnail.ThumbnailsBox();
+        this._thumbnailsSlider = new ThumbnailsSlider(this._thumbnailsBox);
 
-        this._indicator = new MessagesIndicator(viewSelector);
+        this.viewSelector = new ViewSelector.ViewSelector(searchEntry,
+                                                          this.dash.showAppsButton);
+        this.viewSelector.connect('page-changed', Lang.bind(this, this._setVisibility));
+        this.viewSelector.connect('page-empty', Lang.bind(this, this._onPageEmpty));
+
+        this._indicator = new MessagesIndicator(this.viewSelector);
         this.indicatorActor = this._indicator.actor;
 
-        this._viewSelector = viewSelector;
-        this._viewSelector.connect('page-changed', Lang.bind(this, this._setVisibility));
-        this._viewSelector.connect('page-empty', Lang.bind(this, this._onPageEmpty));
+        this.actor = new St.Widget({ layout_manager: new Clutter.BinLayout(),
+                                     x_expand: true, y_expand: true,
+                                     clip_to_allocation: true });
+        this._group = new St.BoxLayout({ name: 'overview-group',
+                                        reactive: true,
+                                        x_expand: true, y_expand: true });
+        this.actor.add_actor(this._group);
+
+        this.actor.add_actor(this._dashSlider.actor);
+
+        this._group.add_actor(this._dashSpacer);
+        this._group.add(this.viewSelector.actor, { x_fill: true,
+                                                   expand: true });
+        this._group.add_actor(this._thumbnailsSlider.actor);
+
+        this._group.connect('notify::allocation', Lang.bind(this, this._updateWorkspacesGeometry));
 
         Main.overview.connect('showing', Lang.bind(this, this._updateSpacerVisibility));
         Main.overview.connect('item-drag-begin', Lang.bind(this,
             function() {
-                let activePage = this._viewSelector.getActivePage();
+                let activePage = this.viewSelector.getActivePage();
                 if (activePage != ViewSelector.ViewPage.WINDOWS)
-                    this._viewSelector.fadeHalf();
+                    this.viewSelector.fadeHalf();
             }));
         Main.overview.connect('item-drag-end', Lang.bind(this,
             function() {
-                this._viewSelector.fadeIn();
+                this.viewSelector.fadeIn();
             }));
         Main.overview.connect('item-drag-cancelled', Lang.bind(this,
             function() {
-                this._viewSelector.fadeIn();
+                this.viewSelector.fadeIn();
             }));
     },
 
+    _updateWorkspacesGeometry: function() {
+        let [x, y] = this.actor.get_transformed_position();
+        let [width, height] = this.actor.get_transformed_size();
+        let geometry = { x: x, y: y, width: width, height: height };
+
+        let spacing = this.actor.get_theme_node().get_length('spacing');
+        let dashWidth = this._dashSlider.getVisibleWidth() + spacing;
+        let thumbnailsWidth = this._thumbnailsSlider.getNonExpandedWidth() + spacing;
+
+        geometry.width -= dashWidth;
+        geometry.width -= thumbnailsWidth;
+
+        if (this.actor.get_text_direction() == Clutter.TextDirection.LTR)
+            geometry.x += dashWidth;
+        else
+            geometry.x += thumbnailsWidth;
+
+        this.viewSelector.setWorkspacesFullGeometry(geometry);
+    },
+
     _setVisibility: function() {
         // Ignore the case when we're leaving the overview, since
         // actors will be made visible again when entering the overview
@@ -521,7 +568,7 @@ const ControlsManager = new Lang.Class({
             (Main.overview.animationInProgress && !Main.overview.visibleTarget))
             return;
 
-        let activePage = this._viewSelector.getActivePage();
+        let activePage = this.viewSelector.getActivePage();
         let dashVisible = (activePage == ViewSelector.ViewPage.WINDOWS ||
                            activePage == ViewSelector.ViewPage.APPS);
         let thumbnailsVisible = (activePage == ViewSelector.ViewPage.WINDOWS);
@@ -541,8 +588,8 @@ const ControlsManager = new Lang.Class({
         if (Main.overview.animationInProgress && !Main.overview.visibleTarget)
             return;
 
-        let activePage = this._viewSelector.getActivePage();
-        this.dashSpacer.visible = (activePage == ViewSelector.ViewPage.WINDOWS);
+        let activePage = this.viewSelector.getActivePage();
+        this._dashSpacer.visible = (activePage == ViewSelector.ViewPage.WINDOWS);
     },
 
     _onPageEmpty: function() {
