@@ -371,16 +371,13 @@ const AppMenuButton = new Lang.Class({
         this._updateIconBoxClip();
     },
 
-    _syncIcon: function() {
-        let icon = this._targetApp.get_faded_icon(2 * PANEL_ICON_SIZE, this._iconBox.text_direction);
-        this._iconBox.set_child(icon);
-    },
-
     _onIconThemeChanged: function() {
         if (this._iconBox.child == null)
             return;
 
-        this._syncIcon();
+        this._iconBox.child.destroy();
+        let icon = this._targetApp.get_faded_icon(2 * PANEL_ICON_SIZE);
+        this._iconBox.set_child(icon);
     },
 
     _updateIconBoxClip: function() {
@@ -571,9 +568,14 @@ const AppMenuButton = new Lang.Class({
         }
 
         if (targetApp == this._targetApp) {
-            if (targetApp && targetApp.get_state() != Shell.AppState.STARTING) {
+            if (targetApp &&
+                targetApp.get_state() != Shell.AppState.STARTING &&
+                targetApp.get_state() != Shell.AppState.BUSY) {
                 this.stopAnimation();
                 this._maybeSetMenu();
+            } else if (targetApp &&
+                       targetApp.get_state() == Shell.AppState.BUSY) {
+                this.startAnimation();
             }
             return;
         }
@@ -598,13 +600,16 @@ const AppMenuButton = new Lang.Class({
         }
 
         this._targetApp = targetApp;
+        let icon = targetApp.get_faded_icon(2 * PANEL_ICON_SIZE);
+
         this._label.setText(targetApp.get_name());
         this.setName(targetApp.get_name());
 
-        this._syncIcon();
+        this._iconBox.set_child(icon);
         this._iconBox.show();
 
-        if (targetApp.get_state() == Shell.AppState.STARTING)
+        if (targetApp.get_state() == Shell.AppState.STARTING ||
+            targetApp.get_state() == Shell.AppState.BUSY)
             this.startAnimation();
         else
             this._maybeSetMenu();
@@ -924,6 +929,7 @@ const PANEL_ITEM_IMPLEMENTATIONS = {
     'volume': imports.ui.status.volume.Indicator,
     'battery': imports.ui.status.power.Indicator,
     'lockScreen': imports.ui.status.lockScreenMenu.Indicator,
+    'logo': imports.gdm.loginDialog.LogoMenuButton,
     'keyboard': imports.ui.status.keyboard.InputSourceIndicator,
     'powerMenu': imports.gdm.powerMenu.PowerMenuButton,
     'userMenu': imports.ui.userMenu.UserMenuButton
