@@ -19,6 +19,12 @@ const Tweener = imports.ui.tweener;
 
 const SLIDER_SCROLL_STEP = 0.05; /* Slider scrolling step in % */
 
+const Ornament = {
+    NONE: 0,
+    DOT: 1,
+    CHECK: 2,
+};
+
 function _ensureStyle(actor) {
     if (actor.get_children) {
         let children = actor.get_children();
@@ -53,7 +59,9 @@ const PopupBaseMenuItem = new Lang.Class({
         this.actor._delegate = this;
 
         this._children = [];
-        this._dot = null;
+        this._ornament = Ornament.NONE;
+        this._ornamentLabel = new St.Label({ style_class: 'popup-menu-ornament' });
+        this.actor.add_actor(this._ornamentLabel);
         this._columnWidths = null;
         this._spacing = 0;
         this.active = false;
@@ -176,40 +184,24 @@ const PopupBaseMenuItem = new Lang.Class({
         this._removeChild(child);
     },
 
-    setShowDot: function(show) {
-        if (show) {
-            if (this._dot)
-                return;
+    setOrnament: function(ornament) {
+        if (ornament == this._ornament)
+            return;
 
-            this._dot = new St.DrawingArea({ style_class: 'popup-menu-item-dot' });
-            this._dot.connect('repaint', Lang.bind(this, this._onRepaintDot));
-            this.actor.add_actor(this._dot);
-            this.actor.add_accessible_state (Atk.StateType.CHECKED);
-        } else {
-            if (!this._dot)
-                return;
+        this._ornament = ornament;
 
-            this._dot.destroy();
-            this._dot = null;
-            this.actor.remove_accessible_state (Atk.StateType.CHECKED);
+        if (ornament == Ornament.DOT) {
+            this._ornamentLabel.text = '\u2022';
+            this.actor.add_accessible_state(Atk.StateType.CHECKED);
+        } else if (ornament == Ornament.CHECK) {
+            this._ornamentLabel.text = '\u2713';
+            this.actor.add_accessible_state(Atk.StateType.CHECKED);
+        } else if (ornament == Ornament.NONE) {
+            this._ornamentLabel.text = '';
+            this.actor.remove_accessible_state(Atk.StateType.CHECKED);
         }
     },
 
-    _onRepaintDot: function(area) {
-        let cr = area.get_context();
-        let [width, height] = area.get_surface_size();
-        let color = area.get_theme_node().get_foreground_color();
-
-        cr.setSourceRGBA (
-            color.red / 255,
-            color.green / 255,
-            color.blue / 255,
-            color.alpha / 255);
-        cr.arc(width / 2, height / 2, width / 3, 0, 2 * Math.PI);
-        cr.fill();
-        cr.$dispose();
-    },
-
     // This returns column widths in logical order (i.e. from the dot
     // to the image), not in visual order (left to right)
     getColumnWidths: function() {
@@ -280,26 +272,25 @@ const PopupBaseMenuItem = new Lang.Class({
         let height = box.y2 - box.y1;
         let direction = this.actor.get_text_direction();
 
-        if (this._dot) {
-            // The dot is placed outside box
-            // one quarter of padding from the border of the container
-            // (so 3/4 from the inner border)
-            // (padding is box.x1)
-            let dotBox = new Clutter.ActorBox();
-            let dotWidth = Math.round(box.x1 / 2);
+        // The ornament is placed outside box
+        // one quarter of padding from the border of the container
+        // (so 3/4 from the inner border)
+        // (padding is box.x1)
+        let ornamentBox = new Clutter.ActorBox();
+        let ornamentWidth = box.x1;
 
-            if (direction == Clutter.TextDirection.LTR) {
-                dotBox.x1 = Math.round(box.x1 / 4);
-                dotBox.x2 = dotBox.x1 + dotWidth;
-            } else {
-                dotBox.x2 = box.x2 + 3 * Math.round(box.x1 / 4);
-                dotBox.x1 = dotBox.x2 - dotWidth;
-            }
-            dotBox.y1 = Math.round(box.y1 + (height - dotWidth) / 2);
-            dotBox.y2 = dotBox.y1 + dotWidth;
-            this._dot.allocate(dotBox, flags);
+        ornamentBox.x1 = 0;
+        ornamentBox.x2 = ornamentWidth;
+        ornamentBox.y1 = box.y1;
+        ornamentBox.y2 = box.y2;
+
+        if (direction == Clutter.TextDirection.RTL) {
+            ornamentBox.x1 += box.x2;
+            ornamentBox.x2 += box.x2;
         }
 
+        this._ornamentLabel.allocate(ornamentBox, flags);
+
         let x;
         if (direction == Clutter.TextDirection.LTR)
             x = box.x1;
@@ -1475,23 +1466,12 @@ const PopupMenuSection = new Lang.Class({
         this.actor = this.box;
         this.actor._delegate = this;
         this.isOpen = true;
-
-        // an array of externally managed separators
-        this.separators = [];
     },
 
     // deliberately ignore any attempt to open() or close(), but emit the
     // corresponding signal so children can still pick it up
     open: function() { this.emit('open-state-changed', true); },
     close: function() { this.emit('open-state-changed', false); },
-
-    destroy: function() {
-        for (let i = 0; i < this.separators.length; i++)
-            this.separators[i].destroy();
-        this.separators = [];
-
-        this.parent();
-    }
 });
 
 const PopupSubMenuMenuItem = new Lang.Class({
@@ -1808,8 +1788,8 @@ const RemoteMenu = new Lang.Class({
         this.model = model;
         this.actionGroup = actionGroup;
 
-        this._actions = { };
-        this._modelChanged(this.model, 0, 0, this.model.get_n_items(), this);
+        this._actions = {};
+        this._trackMenu(model, this);
 
         this._actionStateChangeId = this.actionGroup.connect('action-state-changed', Lang.bind(this, this._actionStateChanged));
         this._actionEnableChangeId = this.actionGroup.connect('action-enabled-changed', Lang.bind(this, this._actionEnabledChanged));
@@ -1829,39 +1809,8 @@ const RemoteMenu = new Lang.Class({
         this.parent();
     },
 
-    _createMenuItem: function(model, index) {
-        let labelValue = model.get_item_attribute_value(index, Gio.MENU_ATTRIBUTE_LABEL, null);
-        let label = labelValue ? labelValue.deep_unpack() : '';
-        // remove all underscores that are not followed by another underscore
-        label = label.replace(/_([^_])/, '$1');
-
-        let section_link = model.get_item_link(index, Gio.MENU_LINK_SECTION);
-        if (section_link) {
-            let item = new PopupMenuSection();
-            if (label) {
-                let title = new PopupMenuItem(label, { reactive: false,
-                                                       style_class: 'popup-subtitle-menu-item' });
-                item._titleMenuItem = title;
-                title._ignored = true;
-                item.addMenuItem(title);
-            }
-            this._modelChanged(section_link, 0, 0, section_link.get_n_items(), item);
-            return [item, true, ''];
-        }
-
-        let submenu_link = model.get_item_link(index, Gio.MENU_LINK_SUBMENU);
-
-        if (submenu_link) {
-            let item = new PopupSubMenuMenuItem(label);
-            this._modelChanged(submenu_link, 0, 0, submenu_link.get_n_items(), item.menu);
-            return [item, false, ''];
-        }
-
-        let action_id = model.get_item_attribute_value(index, Gio.MENU_ATTRIBUTE_ACTION, null).deep_unpack();
-        if (!this.actionGroup.has_action(action_id)) {
-            // the action may not be there yet, wait for action-added
-            return [null, false, 'action-added'];
-        }
+    _actionAdded: function(model, item, index) {
+        let action_id = item.action_id;
 
         if (!this._actions[action_id])
             this._actions[action_id] = { enabled: this.actionGroup.get_action_enabled(action_id),
@@ -1869,7 +1818,7 @@ const RemoteMenu = new Lang.Class({
                                          items: [ ],
                                        };
         let action = this._actions[action_id];
-        let item, target, destroyId, specificSignalId;
+        let target, destroyId, specificSignalId;
 
         if (action.state) {
             // Docs have get_state_hint(), except that the DBus protocol
@@ -1880,28 +1829,28 @@ const RemoteMenu = new Lang.Class({
 
             switch (String.fromCharCode(action.state.classify())) {
             case 'b':
-                item = new PopupSwitchMenuItem(label, action.state.get_boolean());
                 action.items.push(item);
-                specificSignalId = item.connect('toggled', Lang.bind(this, function(item) {
+                item.setOrnament(action.state.get_boolean() ?
+                                 Ornament.CHECK : Ornament.NONE);
+                specificSignalId = item.connect('activate', Lang.bind(this, function(item) {
                     this.actionGroup.activate_action(action_id, null);
                 }));
                 break;
             case 's':
-                item = new PopupMenuItem(label);
-                item._remoteTarget = model.get_item_attribute_value(index, Gio.MENU_ATTRIBUTE_TARGET, null).deep_unpack();
                 action.items.push(item);
-                item.setShowDot(action.state.deep_unpack() == item._remoteTarget);
+                item._remoteTarget = model.get_item_attribute_value(index, Gio.MENU_ATTRIBUTE_TARGET, null).deep_unpack();
+                item.setOrnament(action.state.deep_unpack() == item._remoteTarget ?
+                                 Ornament.DOT : Ornament.NONE);
                 specificSignalId = item.connect('activate', Lang.bind(this, function(item) {
                     this.actionGroup.activate_action(action_id, GLib.Variant.new_string(item._remoteTarget));
                 }));
                 break;
             default:
                 log('Action "%s" has state of type %s, which is not supported'.format(action_id, action.state.get_type_string()));
-                return [null, false, 'action-state-changed'];
+                return;
             }
         } else {
             target = model.get_item_attribute_value(index, Gio.MENU_ATTRIBUTE_TARGET, null);
-            item = new PopupMenuItem(label);
             action.items.push(item);
             specificSignalId = item.connect('activate', Lang.bind(this, function() {
                 this.actionGroup.activate_action(action_id, target);
@@ -1918,94 +1867,52 @@ const RemoteMenu = new Lang.Class({
             if (pos != -1)
                 action.items.splice(pos, 1);
         }));
+    },
 
-        return [item, false, ''];
-    }, 
+    _trackMenu: function(model, item) {
+        item._tracker = Shell.MenuTracker.new(model,
+                                              null, /* action namespace */
+                                              Lang.bind(this, this._insertItem, item),
+                                              Lang.bind(this, this._removeItem, item));
 
-    _modelChanged: function(model, position, removed, added, target) {
-        let j, k;
-        let j0, k0;
+        item.connect('destroy', function() {
+            item._tracker.destroy();
+            item._tracker = null;
+        });
+    },
 
-        let currentItems = target._getMenuItems();
+    _createMenuItem: function(model, index) {
+        let labelValue = model.get_item_attribute_value(index, Gio.MENU_ATTRIBUTE_LABEL, null);
+        let label = labelValue ? labelValue.deep_unpack() : '';
+        // remove all underscores that are not followed by another underscore
+        label = label.replace(/_([^_])/, '$1');
 
-        k0 = 0;
-        // skip ignored items at the beginning
-        while (k0 < currentItems.length && currentItems[k0]._ignored)
-            k0++;
-        // find the right menu item matching the model item
-        for (j0 = 0; k0 < currentItems.length && j0 < position; j0++, k0++) {
-            if (currentItems[k0]._ignored)
-                k0++;
+        let submenuModel = model.get_item_link(index, Gio.MENU_LINK_SUBMENU);
+        if (submenuModel) {
+            let item = new PopupSubMenuMenuItem(label);
+            this._trackMenu(submenuModel, item.menu);
+            return item;
         }
 
-        if (removed == -1) {
-            // special flag to indicate we should destroy everything
-            for (k = k0; k < currentItems.length; k++)
-                currentItems[k].destroy();
-        } else {
-            for (j = j0, k = k0; k < currentItems.length && j < j0 + removed; j++, k++) {
-                currentItems[k].destroy();
+        let item = new PopupMenuItem(label);
+        let action_id = model.get_item_attribute_value(index, Gio.MENU_ATTRIBUTE_ACTION, null).deep_unpack();
+        item.actor.can_focus = item.actor.reactive = false;
 
-                if (currentItems[k]._ignored)
-                    j--;
-            }
+        item.action_id = action_id;
+
+        if (this.actionGroup.has_action(action_id)) {
+            this._actionAdded(model, item, index);
+            return item;
         }
 
-        for (j = j0, k = k0; j < j0 + added; j++, k++) {
-            let [item, addSeparator, changeSignal] = this._createMenuItem(model, j);
+        let signalId = this.actionGroup.connect('action-added', Lang.bind(this, function(actionGroup, actionName) {
+            actionGroup.disconnect(signalId);
+            if (this._actions[actionName]) return;
 
-            if (item) {
-                // separators must be added in the parent to make autohiding work
-                if (addSeparator) {
-                    let separator = new PopupSeparatorMenuItem();
-                    item.separators.push(separator);
-                    separator._ignored = true;
-                    target.addMenuItem(separator, k+1);
-                    k++;
-                }
+            this._actionAdded(model, item, index);
+        }));
 
-                target.addMenuItem(item, k);
-
-                if (addSeparator) {
-                    let separator = new PopupSeparatorMenuItem();
-                    item.separators.push(separator);
-                    separator._ignored = true;
-                    target.addMenuItem(separator, k+1);
-                    k++;
-                }
-            } else if (changeSignal) {
-                let signalId = this.actionGroup.connect(changeSignal, Lang.bind(this, function(actionGroup, actionName) {
-                    actionGroup.disconnect(signalId);
-                    if (this._actions[actionName]) return;
-
-                    // force a full update
-                    this._modelChanged(model, 0, -1, model.get_n_items(), target);
-                }));
-            }
-        }
-
-        if (!model._changedId) {
-            model._changedId = model.connect('items-changed', Lang.bind(this, this._modelChanged, target));
-            model._destroyId = target.connect('destroy', function() {
-                if (model._changedId)
-                    model.disconnect(model._changedId);
-                if (model._destroyId)
-                    target.disconnect(model._destroyId);
-                model._changedId = 0;
-                model._destroyId = 0;
-            });
-        }
-
-        if (target instanceof PopupMenuSection) {
-            if (target._titleMenuItem)
-                target.actor.visible = target.numMenuItems != 1;
-            else
-                target.actor.visible = target.numMenuItems != 0;
-        } else {
-            let sourceItem = target.sourceActor._delegate;
-            if (sourceItem instanceof PopupSubMenuMenuItem)
-                sourceItem.actor.visible = target.numMenuItems != 0;
-        }
+        return item;
     },
 
     _actionStateChanged: function(actionGroup, action_id) {
@@ -2018,7 +1925,8 @@ const RemoteMenu = new Lang.Class({
             switch (String.fromCharCode(action.state.classify())) {
             case 'b':
                 for (let i = 0; i < action.items.length; i++)
-                    action.items[i].setToggleState(action.state.get_boolean());
+                    action.items[i].setOrnament(action.state.get_boolean() ?
+                                                Ornament.CHECK : Ornament.NONE);
                 break;
             case 'd':
                 for (let i = 0; i < action.items.length; i++)
@@ -2026,7 +1934,8 @@ const RemoteMenu = new Lang.Class({
                 break;
             case 's':
                 for (let i = 0; i < action.items.length; i++)
-                    action.items[i].setShowDot(action.items[i]._remoteTarget == action.state.deep_unpack());
+                    action.items[i].setOrnament(action.items[i]._remoteTarget == action.state.deep_unpack() ?
+                                                Ornament.DOT : Ornament.NONE);
             }
         }
     },
@@ -2043,7 +1952,23 @@ const RemoteMenu = new Lang.Class({
                 item.actor.reactive = item.actor.can_focus = action.enabled;
             }
         }
-    }
+    },
+
+    _insertItem: function(position, model, item_index, action_namespace, is_separator, target) {
+        let item;
+
+        if (is_separator)
+            item = new PopupSeparatorMenuItem();
+        else
+            item = this._createMenuItem(model, item_index);
+
+        target.addMenuItem(item, position);
+    },
+
+    _removeItem: function(position, target) {
+        let items = target._getMenuItems();
+        items[position].destroy();
+    },
 });
 
 /* Basic implementation of a menu manager.
