@@ -16,7 +16,6 @@ const _modes = {
     'restrictive': {
         parentMode: null,
         stylesheetName: 'gnome-shell.css',
-        overridesSchema: 'org.gnome.shell.overrides',
         hasOverview: false,
         showCalendarEvents: false,
         allowSettings: false,
@@ -46,7 +45,7 @@ const _modes = {
         unlockDialog: imports.gdm.loginDialog.LoginDialog,
         components: ['polkitAgent'],
         panel: {
-            left: [],
+            left: ['logo'],
             center: ['dateMenu'],
             right: ['a11yGreeter', 'display', 'keyboard',
                     'volume', 'battery', 'powerMenu']
@@ -85,7 +84,7 @@ const _modes = {
         components: ['networkAgent', 'keyring'],
         panel: {
             left: [],
-            center: ['dateMenu'],
+            center: [],
             right: ['a11yGreeter', 'keyboard', 'volume', 'battery']
         }
     },
@@ -196,10 +195,6 @@ const SessionMode = new Lang.Class({
         return this._modeStack[this._modeStack.length - 1];
     },
 
-    get allowScreencast() {
-        return this.components.indexOf('recorder') != -1;
-    },
-
     _sync: function() {
         let params = this._modes[this.currentMode];
         let defaults;
