@@ -33,33 +33,6 @@ const KEY_INPUT_SOURCES = 'sources';
 const INPUT_SOURCE_TYPE_XKB = 'xkb';
 const INPUT_SOURCE_TYPE_IBUS = 'ibus';
 
-// This is the longest we'll keep the keyboard frozen until an input
-// source is active.
-const MAX_INPUT_SOURCE_ACTIVATION_TIME = 4000; // ms
-
-const BUS_NAME = 'org.gnome.SettingsDaemon.Keyboard';
-const OBJECT_PATH = '/org/gnome/SettingsDaemon/Keyboard';
-
-const KeyboardManagerInterface =
-<interface name="org.gnome.SettingsDaemon.Keyboard">
-<method name="SetInputSource">
-    <arg type="u" direction="in" />
-</method>
-</interface>;
-
-const KeyboardManagerProxy = Gio.DBusProxy.makeProxyWrapper(KeyboardManagerInterface);
-
-function releaseKeyboard() {
-    if (Main.modalCount > 0)
-        global.display.unfreeze_keyboard(global.get_current_time());
-    else
-        global.display.ungrab_keyboard(global.get_current_time());
-}
-
-function holdKeyboard() {
-    global.freeze_keyboard(global.get_current_time());
-}
-
 const IBusManager = new Lang.Class({
     Name: 'IBusManager',
 
@@ -72,24 +45,26 @@ const IBusManager = new Lang.Class({
         this._readyCallback = readyCallback;
         this._candidatePopup = new IBusCandidatePopup.CandidatePopup();
 
+        this._ibus = null;
         this._panelService = null;
         this._engines = {};
         this._ready = false;
         this._registerPropertiesId = 0;
         this._currentEngineName = null;
 
-        this._ibus = IBus.Bus.new_async();
-        this._ibus.connect('connected', Lang.bind(this, this._onConnected));
-        this._ibus.connect('disconnected', Lang.bind(this, this._clear));
-        // Need to set this to get 'global-engine-changed' emitions
-        this._ibus.set_watch_ibus_signal(true);
-        this._ibus.connect('global-engine-changed', Lang.bind(this, this._engineChanged));
+        this._nameWatcherId = Gio.DBus.session.watch_name(IBus.SERVICE_IBUS,
+                                                          Gio.BusNameWatcherFlags.NONE,
+                                                          Lang.bind(this, this._onNameAppeared),
+                                                          Lang.bind(this, this._clear));
     },
 
     _clear: function() {
         if (this._panelService)
             this._panelService.destroy();
+        if (this._ibus)
+            this._ibus.destroy();
 
+        this._ibus = null;
         this._panelService = null;
         this._candidatePopup.setPanelService(null);
         this._engines = {};
@@ -101,12 +76,18 @@ const IBusManager = new Lang.Class({
             this._readyCallback(false);
     },
 
+    _onNameAppeared: function() {
+        this._ibus = IBus.Bus.new_async();
+        this._ibus.connect('connected', Lang.bind(this, this._onConnected));
+    },
+
     _onConnected: function() {
         this._ibus.list_engines_async(-1, null, Lang.bind(this, this._initEngines));
         this._ibus.request_name_async(IBus.SERVICE_PANEL,
                                       IBus.BusNameFlag.REPLACE_EXISTING,
                                       -1, null,
                                       Lang.bind(this, this._initPanelService));
+        this._ibus.connect('disconnected', Lang.bind(this, this._clear));
     },
 
     _initEngines: function(ibus, result) {
@@ -128,6 +109,9 @@ const IBusManager = new Lang.Class({
             this._panelService = new IBus.PanelService({ connection: this._ibus.get_connection(),
                                                          object_path: IBus.PATH_PANEL });
             this._candidatePopup.setPanelService(this._panelService);
+            // Need to set this to get 'global-engine-changed' emitions
+            this._ibus.set_watch_ibus_signal(true);
+            this._ibus.connect('global-engine-changed', Lang.bind(this, this._engineChanged));
             this._panelService.connect('update-property', Lang.bind(this, this._updateProperty));
             // If an engine is already active we need to get its properties
             this._ibus.get_global_engine_async(-1, null, Lang.bind(this, function(i, result) {
@@ -156,9 +140,6 @@ const IBusManager = new Lang.Class({
     },
 
     _engineChanged: function(bus, engineName) {
-        if (!this._ready)
-            return;
-
         this._currentEngineName = engineName;
 
         if (this._registerPropertiesId != 0)
@@ -356,14 +337,14 @@ const InputSourceIndicator = new Lang.Class({
             Main.wm.addKeybinding('switch-input-source',
                                   new Gio.Settings({ schema: "org.gnome.desktop.wm.keybindings" }),
                                   Meta.KeyBindingFlags.REVERSES,
-                                  Shell.KeyBindingMode.ALL,
+                                  Shell.KeyBindingMode.ALL & ~Shell.KeyBindingMode.MESSAGE_TRAY,
                                   Lang.bind(this, this._switchInputSource));
         this._keybindingActionBackward =
             Main.wm.addKeybinding('switch-input-source-backward',
                                   new Gio.Settings({ schema: "org.gnome.desktop.wm.keybindings" }),
                                   Meta.KeyBindingFlags.REVERSES |
                                   Meta.KeyBindingFlags.REVERSED,
-                                  Shell.KeyBindingMode.ALL,
+                                  Shell.KeyBindingMode.ALL & ~Shell.KeyBindingMode.MESSAGE_TRAY,
                                   Lang.bind(this, this._switchInputSource));
         this._settings = new Gio.Settings({ schema: DESKTOP_INPUT_SOURCES_SCHEMA });
         this._settings.connect('changed::' + KEY_CURRENT_INPUT_SOURCE, Lang.bind(this, this._currentInputSourceChanged));
@@ -383,15 +364,6 @@ const InputSourceIndicator = new Lang.Class({
         this._ibusManager.connect('property-updated', Lang.bind(this, this._ibusPropertyUpdated));
         this._inputSourcesChanged();
 
-        this._keyboardManager = new KeyboardManagerProxy(Gio.DBus.session, BUS_NAME, OBJECT_PATH,
-                                                         function(proxy, error) {
-                                                             if (error)
-                                                                 log(error.message);
-                                                         });
-        this._keyboardManager.g_default_timeout = MAX_INPUT_SOURCE_ACTIVATION_TIME;
-
-        global.display.connect('modifiers-accelerator-activated', Lang.bind(this, this._modifiersSwitcher));
-
         this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
         this._showLayoutItem = this.menu.addAction(_("Show Keyboard Layout"), Lang.bind(this, this._showLayout));
 
@@ -425,43 +397,10 @@ const InputSourceIndicator = new Lang.Class({
         this._inputSourcesChanged();
     },
 
-    _modifiersSwitcher: function() {
-        let sourceIndexes = Object.keys(this._inputSources);
-        if (sourceIndexes.length == 0) {
-            releaseKeyboard();
-            return true;
-        }
-
-        let is = this._currentSource;
-        if (!is)
-            is = this._inputSources[sourceIndexes[0]];
-
-        let nextIndex = is.index + 1;
-        if (nextIndex > sourceIndexes[sourceIndexes.length - 1])
-            nextIndex = 0;
-
-        while (!(is = this._inputSources[nextIndex]))
-            nextIndex += 1;
-
-        is.activate();
-        return true;
-    },
-
     _switchInputSource: function(display, screen, window, binding) {
         if (this._mruSources.length < 2)
             return;
 
-        // HACK: Fall back on simple input source switching since we
-        // can't show a popup switcher while a GrabHelper grab is in
-        // effect without considerable work to consolidate the usage
-        // of pushModal/popModal and grabHelper. See
-        // https://bugzilla.gnome.org/show_bug.cgi?id=695143 .
-        if (Main.keybindingMode == Shell.KeyBindingMode.MESSAGE_TRAY ||
-            Main.keybindingMode == Shell.KeyBindingMode.TOPBAR_POPUP) {
-            this._modifiersSwitcher();
-            return;
-        }
-
         let popup = new InputSourcePopup(this._mruSources, this._keybindingAction, this._keybindingActionBackward);
         let modifiers = binding.get_modifiers();
         let backwards = modifiers & Meta.VirtualModifier.SHIFT_MASK;
@@ -478,7 +417,7 @@ const InputSourceIndicator = new Lang.Class({
         [oldSource, this._currentSource] = [this._currentSource, newSource];
 
         if (oldSource) {
-            oldSource.menuItem.setShowDot(false);
+            oldSource.menuItem.setOrnament(PopupMenu.Ornament.NONE);
             oldSource.indicatorLabel.hide();
         }
 
@@ -496,7 +435,7 @@ const InputSourceIndicator = new Lang.Class({
 
         this.actor.show();
 
-        newSource.menuItem.setShowDot(true);
+        newSource.menuItem.setOrnament(PopupMenu.Ornament.DOT);
         newSource.indicatorLabel.show();
 
         this._buildPropSection(newSource.properties);
@@ -549,8 +488,10 @@ const InputSourceIndicator = new Lang.Class({
             let is = new InputSource(type, id, displayName, shortName, i);
 
             is.connect('activate', Lang.bind(this, function() {
-                holdKeyboard();
-                this._keyboardManager.SetInputSourceRemote(is.index, releaseKeyboard);
+                if (this._currentSource && this._currentSource.index == is.index)
+                    return;
+                this._settings.set_value(KEY_CURRENT_INPUT_SOURCE,
+                                         GLib.Variant.new_uint32(is.index));
             }));
 
             if (!(is.shortName in inputSourcesByShortName))
@@ -720,7 +661,8 @@ const InputSourceIndicator = new Lang.Class({
                 item.prop = prop;
                 radioGroup.push(item);
                 item.radioGroup = radioGroup;
-                item.setShowDot(prop.get_state() == IBus.PropState.CHECKED);
+                item.setOrnament(prop.get_state() == IBus.PropState.CHECKED ?
+                                 PopupMenu.Ornament.DOT : PopupMenu.Ornament.NONE);
                 item.connect('activate', Lang.bind(this, function() {
                     if (item.prop.get_state() == IBus.PropState.CHECKED)
                         return;
@@ -728,12 +670,12 @@ const InputSourceIndicator = new Lang.Class({
                     let group = item.radioGroup;
                     for (let i = 0; i < group.length; ++i) {
                         if (group[i] == item) {
-                            item.setShowDot(true);
+                            item.setOrnament(PopupMenu.Ornament.DOT);
                             item.prop.set_state(IBus.PropState.CHECKED);
                             this._ibusManager.activateProperty(item.prop.get_key(),
                                                                IBus.PropState.CHECKED);
                         } else {
-                            group[i].setShowDot(false);
+                            group[i].setOrnament(PopupMenu.Ornament.NONE);
                             group[i].prop.set_state(IBus.PropState.UNCHECKED);
                             this._ibusManager.activateProperty(group[i].prop.get_key(),
                                                                IBus.PropState.UNCHECKED);
