@@ -5,17 +5,10 @@ const Gio = imports.gi.Gio;
 const Lang = imports.lang;
 const NetworkManager = imports.gi.NetworkManager;
 const NMClient = imports.gi.NMClient;
+const NMGtk = imports.gi.NMGtk;
 const Signals = imports.signals;
 const St = imports.gi.St;
 
-// Some of the new code depends on as-yet-unreleased NM
-var NMGtk;
-try {
-    NMGtk = imports.gi.NMGtk;
-} catch(e) {
-    NMGtk = null;
-}
-
 const Main = imports.ui.main;
 const PanelMenu = imports.ui.panelMenu;
 const PopupMenu = imports.ui.popupMenu;
@@ -34,7 +27,6 @@ const NMConnectionCategory = {
 };
 
 const NMAccessPointSecurity = {
-    UNKNOWN: 0,
     NONE: 1,
     WEP: 2,
     WPA_PSK: 3,
@@ -52,20 +44,6 @@ const NM80211ApSecurityFlags = NetworkManager['80211ApSecurityFlags'];
 // (the remaining are placed into More…)
 const NUM_VISIBLE_NETWORKS = 5;
 
-function macToArray(string) {
-    return string.split(':').map(function(el) {
-        return parseInt(el, 16);
-    });
-}
-
-function macCompare(one, two) {
-    for (let i = 0; i < 6; i++) {
-        if (one[i] != two[i])
-            return false;
-    }
-    return true;
-}
-
 function ssidCompare(one, two) {
     if (!one || !two)
         return false;
@@ -130,8 +108,7 @@ const NMNetworkMenuItem = new Lang.Class({
         this._icons.add_actor(this._signalIcon);
 
         this._secureIcon = new St.Icon({ style_class: 'popup-menu-icon' });
-        if (this.bestAP._secType != NMAccessPointSecurity.UNKNOWN &&
-            this.bestAP._secType != NMAccessPointSecurity.NONE)
+        if (this.bestAP._secType != NMAccessPointSecurity.NONE)
             this._secureIcon.icon_name = 'network-wireless-encrypted-symbolic';
         this._icons.add_actor(this._secureIcon);
     },
@@ -153,18 +130,12 @@ const NMWirelessSectionTitleMenuItem = new Lang.Class({
     Name: 'NMWirelessSectionTitleMenuItem',
     Extends: PopupMenu.PopupSwitchMenuItem,
 
-    _init: function(client, property, title, params) {
-        params = params || { };
-        params.style_class = 'popup-subtitle-menu-item';
-        this.parent(title, false, params);
+    _init: function(client) {
+        this.parent(_("Wi-Fi"), false, { style_class: 'popup-subtitle-menu-item' });
 
         this._client = client;
-        this._property = property + '_enabled';
-        this._propertyHardware = property + '_hardware_enabled';
-        this._setEnabledFunc = property + '_set_enabled';
-
-        this._client.connect('notify::' + property + '-enabled', Lang.bind(this, this._propertyChanged));
-        this._client.connect('notify::' + property + '-hardware-enabled', Lang.bind(this, this._propertyChanged));
+        this._client.connect('notify::wireless-enabled', Lang.bind(this, this._propertyChanged));
+        this._client.connect('notify::wireless-hardware-enabled', Lang.bind(this, this._propertyChanged));
 
         this._propertyChanged();
     },
@@ -186,12 +157,12 @@ const NMWirelessSectionTitleMenuItem = new Lang.Class({
     activate: function(event) {
         this.parent(event);
 
-        this._client[this._setEnabledFunc](this._switch.state);
+        this._client.wireless_set_enabled(this._switch.state);
     },
 
     _propertyChanged: function() {
-        this._softwareEnabled = this._client[this._property];
-        this._hardwareEnabled = this._client[this._propertyHardware];
+        this._softwareEnabled = this._client.wireless_enabled;
+        this._hardwareEnabled = this._client.wireless_hardware_enabled;
 
         let enabled = this._softwareEnabled && this._hardwareEnabled;
         this.setToggleState(enabled);
@@ -315,7 +286,6 @@ const NMDevice = new Lang.Class({
 
         this._activeConnection = null;
         this._activeConnectionItem = null;
-        this._autoConnectionItem = null;
         this._overflowItem = null;
 
         this.statusItem = new PopupMenu.PopupSwitchMenuItem('', this.connected, { style_class: 'popup-subtitle-menu-item' });
@@ -402,13 +372,9 @@ const NMDevice = new Lang.Class({
     },
 
     _activateAutomaticConnection: function() {
-        let connection = this._createAutomaticConnection();
-        if (connection) {
-            this._client.add_and_activate_connection(connection, this.device, null, null);
-            return true;
-        }
-
-        return false;
+        let connection = new NetworkManager.Connection();
+        this._client.add_and_activate_connection(connection, this.device, null, null);
+        return true;
     },
 
     get connected() {
@@ -502,11 +468,6 @@ const NMDevice = new Lang.Class({
             this.statusItem.label.text = this.device._description;
     },
 
-    // protected
-    _createAutomaticConnection: function() {
-        throw new TypeError('Invoking pure virtual function NMDevice.createAutomaticConnection');
-    },
-
     _queueCreateSection: function() {
         if (this._deferredWorkId) {
             this._clearSection();
@@ -517,7 +478,6 @@ const NMDevice = new Lang.Class({
     _clearSection: function() {
         // Clear everything
         this.section.removeAll();
-        this._autoConnectionItem = null;
         this._activeConnectionItem = null;
         this._overflowItem = null;
         for (let i = 0; i < this._connections.length; i++) {
@@ -556,14 +516,6 @@ const NMDevice = new Lang.Class({
                 } else
                     this.section.addMenuItem(obj.item);
             }
-        } else if (this._autoConnectionName) {
-            this._autoConnectionItem = new PopupMenu.PopupMenuItem(this._autoConnectionName);
-            this._autoConnectionItem.connect('activate', Lang.bind(this, function() {
-                let connection = this._createAutomaticConnection();
-                if (connection)
-                    this._client.add_and_activate_connection(connection, this.device, null, null);
-            }));
-            this.section.addMenuItem(this._autoConnectionItem);
         }
     },
 
@@ -588,7 +540,7 @@ const NMDevice = new Lang.Class({
             title = _("Connected (private)");
         }
         this._activeConnectionItem = new PopupMenu.PopupMenuItem(title, { reactive: false });
-        this._activeConnectionItem.setShowDot(true);
+        this._activeConnectionItem.setOrnament(PopupMenu.Ornament.DOT);
     },
 
     _deviceStateChanged: function(device, newstate, oldstate, reason) {
@@ -665,24 +617,10 @@ const NMDeviceWired = new Lang.Class({
 
     _init: function(client, device, connections) {
         device._description = _("Wired");
-        this._autoConnectionName = _("Auto Ethernet");
         this.category = NMConnectionCategory.WIRED;
 
         this.parent(client, device, connections);
     },
-
-    _createAutomaticConnection: function() {
-        let connection = new NetworkManager.Connection();
-        let uuid = NetworkManager.utils_uuid_generate();
-        connection.add_setting(new NetworkManager.SettingWired());
-        connection.add_setting(new NetworkManager.SettingConnection({
-            uuid: uuid,
-            id: this._autoConnectionName,
-            type: NetworkManager.SETTING_WIRED_SETTING_NAME,
-            autoconnect: true
-        }));
-        return connection;
-    }
 });
 
 const NMDeviceModem = new Lang.Class({
@@ -723,13 +661,10 @@ const NMDeviceModem = new Lang.Class({
             this._connectionType = NetworkManager.SETTING_GSM_SETTING_NAME;
         }
 
-        if (is_wwan) {
+        if (is_wwan)
             this.category = NMConnectionCategory.WWAN;
-            this._autoConnectionName = _("Auto broadband");
-        } else {
+        else
             this.category = NMConnectionCategory.WIRED;
-            this._autoConnectionName = _("Auto dial-up");
-        }
 
         if (this.mobileDevice) {
             this._operatorNameId = this.mobileDevice.connect('notify::operator-name', Lang.bind(this, function() {
@@ -824,50 +759,20 @@ const NMDeviceBluetooth = new Lang.Class({
 
     _init: function(client, device, connections) {
         device._description = _("Bluetooth");
-        this._autoConnectionName = this._makeConnectionName(device);
-        device.connect('notify::name', Lang.bind(this, this._updateAutoConnectionName));
 
         this.category = NMConnectionCategory.WWAN;
 
         this.parent(client, device, connections);
     },
 
-    _createAutomaticConnection: function() {
-        let connection = new NetworkManager.Connection;
-        let uuid = NetworkManager.utils_uuid_generate();
-        connection.add_setting(new NetworkManager.SettingBluetooth);
-        connection.add_setting(new NetworkManager.SettingConnection({
-            uuid: uuid,
-            id: this._autoConnectionName,
-            type: NetworkManager.SETTING_BLUETOOTH_SETTING_NAME,
-            autoconnect: false
-        }));
-        return connection;
-    },
-
     _activateAutomaticConnection: function() {
         // FIXME: DUN devices are configured like modems, so
-        // we need to spawn the mobile wizard
+        // We need to spawn the mobile wizard
         // but the network panel doesn't support bluetooth at the moment
         // so we just create an empty connection and hope
         // that this phone supports PAN
 
         return this.parent();
-    },
-
-    _makeConnectionName: function(device) {
-        let name = device.name;
-        if (name)
-            return _("Auto %s").format(name);
-        else
-            return _("Auto bluetooth");
-    },
-
-    _updateAutoConnectionName: function() {
-        this._autoConnectionName = this._makeConnectionName(this.device);
-
-        this._queueCreateSection();
-        this._updateStatusItem();
     }
 });
 
@@ -903,20 +808,20 @@ const NMDeviceWireless = new Lang.Class({
             }
 
             let pos = this._findNetwork(ap);
-            let obj;
+            let network;
             if (pos != -1) {
-                obj = this._networks[pos];
-                obj.accessPoints.push(ap);
+                network = this._networks[pos];
+                network.accessPoints.push(ap);
             } else {
-                obj = { ssid: ap.get_ssid(),
-                        mode: ap.mode,
-                        security: this._getApSecurityType(ap),
-                        connections: [ ],
-                        item: null,
-                        accessPoints: [ ap ]
+                network = { ssid: ap.get_ssid(),
+                            mode: ap.mode,
+                            security: this._getApSecurityType(ap),
+                            connections: [ ],
+                            item: null,
+                            accessPoints: [ ap ]
                       };
-                obj.ssidText = ssidToLabel(obj.ssid);
-                this._networks.push(obj);
+                network.ssidText = ssidToLabel(network.ssid);
+                this._networks.push(network);
             }
             ap._updateId = ap.connect('notify::strength', Lang.bind(this, this._onApStrengthChanged));
 
@@ -924,8 +829,8 @@ const NMDeviceWireless = new Lang.Class({
             for (let j = 0; j < validConnections.length; j++) {
                 let connection = validConnections[j];
                 if (ap.connection_valid(connection) &&
-                    obj.connections.indexOf(connection) == -1) {
-                    obj.connections.push(connection);
+                    network.connections.indexOf(connection) == -1) {
+                    network.connections.push(connection);
                 }
             }
         }
@@ -934,16 +839,7 @@ const NMDeviceWireless = new Lang.Class({
         for (let i = 0; i < this._networks.length; i++)
             sortAccessPoints(this._networks[i].accessPoints);
 
-        if (this.device.active_access_point) {
-            let networkPos = this._findNetwork(this.device.active_access_point);
-
-            if (networkPos == -1) // the connected access point is invisible
-                this._activeNetwork = null;
-            else
-                this._activeNetwork = this._networks[networkPos];
-        } else {
-            this._activeNetwork = null;
-        }
+        this._activeApChanged();
         this._networks.sort(this._networkSortFunction);
 
         this._apChangedId = device.connect('notify::active-access-point', Lang.bind(this, this._activeApChanged));
@@ -1090,9 +986,9 @@ const NMDeviceWireless = new Lang.Class({
 
     _findExistingNetwork: function(accessPoint) {
         for (let i = 0; i < this._networks.length; i++) {
-            let apObj = this._networks[i];
-            for (let j = 0; j < apObj.accessPoints.length; j++) {
-                if (apObj.accessPoints[j] == accessPoint)
+            let network = this._networks[i];
+            for (let j = 0; j < network.accessPoints.length; j++) {
+                if (network.accessPoints[j] == accessPoint)
                     return { network: i, ap: j };
             }
         }
@@ -1140,30 +1036,30 @@ const NMDeviceWireless = new Lang.Class({
         }
 
         let pos = this._findNetwork(accessPoint);
-        let apObj;
+        let network;
         let needsupdate = false;
 
         if (pos != -1) {
-            apObj = this._networks[pos];
-            if (apObj.accessPoints.indexOf(accessPoint) != -1) {
+            network = this._networks[pos];
+            if (network.accessPoints.indexOf(accessPoint) != -1) {
                 log('Access point was already seen, not adding again');
                 return;
             }
 
-            Util.insertSorted(apObj.accessPoints, accessPoint, function(one, two) {
+            Util.insertSorted(network.accessPoints, accessPoint, function(one, two) {
                 return two.strength - one.strength;
             });
-            if (apObj.item)
-                apObj.item.updateBestAP(apObj.accessPoints[0]);
+            if (network.item)
+                network.item.updateBestAP(network.accessPoints[0]);
         } else {
-            apObj = { ssid: accessPoint.get_ssid(),
-                      mode: accessPoint.mode,
-                      security: this._getApSecurityType(accessPoint),
-                      connections: [ ],
-                      item: null,
-                      accessPoints: [ accessPoint ]
-                    };
-            apObj.ssidText = ssidToLabel(apObj.ssid);
+            network = { ssid: accessPoint.get_ssid(),
+                        mode: accessPoint.mode,
+                        security: this._getApSecurityType(accessPoint),
+                        connections: [ ],
+                        item: null,
+                        accessPoints: [ accessPoint ]
+                      };
+            network.ssidText = ssidToLabel(network.ssid);
         }
         accessPoint._updateId = accessPoint.connect('notify::strength', Lang.bind(this, this._onApStrengthChanged));
 
@@ -1171,14 +1067,14 @@ const NMDeviceWireless = new Lang.Class({
         for (let i = 0; i < this._connections.length; i++) {
             let connection = this._connections[i].connection;
             if (accessPoint.connection_valid(connection) &&
-                apObj.connections.indexOf(connection) == -1) {
-                apObj.connections.push(connection);
+                network.connections.indexOf(connection) == -1) {
+                network.connections.push(connection);
             }
         }
 
         if (pos != -1)
             this._networks.splice(pos, 1);
-        let newPos = Util.insertSorted(this._networks, apObj, this._networkSortFunction);
+        let newPos = Util.insertSorted(this._networks, network, this._networkSortFunction);
 
         // Queue an update of the UI if we changed the order
         if (newPos != pos)
@@ -1198,28 +1094,28 @@ const NMDeviceWireless = new Lang.Class({
             return;
         }
 
-        let apObj = this._networks[res.network];
-        apObj.accessPoints.splice(res.ap, 1);
+        let network = this._networks[res.network];
+        network.accessPoints.splice(res.ap, 1);
 
-        if (apObj.accessPoints.length == 0) {
-            if (this._activeNetwork == apObj)
+        if (network.accessPoints.length == 0) {
+            if (this._activeNetwork == network)
                 this._activeNetwork = null;
 
-            if (apObj.item)
-                apObj.item.destroy();
+            if (network.item)
+                network.item.destroy();
 
             if (this._overflowItem) {
-                if (!apObj.isMore) {
+                if (!network.isMore) {
                     // we removed an item in the main menu, and we have a more submenu
                     // we need to extract the first item in more and move it to the submenu
 
                     let item = this._overflowItem.menu.firstMenuItem;
-                    if (item && item._apObj) {
+                    if (item && item._network) {
                         item.destroy();
                         // clear the cycle, and allow the construction of the new item
-                        item._apObj.item = null;
+                        item._network.item = null;
 
-                        this._createNetworkItem(item._apObj, NUM_VISIBLE_NETWORKS-1);
+                        this._createNetworkItem(item._network, NUM_VISIBLE_NETWORKS-1);
                     } else {
                         log('The more... menu was existing and empty! This should not happen');
                     }
@@ -1238,14 +1134,14 @@ const NMDeviceWireless = new Lang.Class({
             let okPrev = true, okNext = true;
 
             if (res.network > 0)
-                okPrev = this._networkSortFunction(this._networks[res.network - 1], apObj) >= 0;
+                okPrev = this._networkSortFunction(this._networks[res.network - 1], network) >= 0;
             if (res.network < this._networks.length-1)
-                okNext = this._networkSortFunction(this._networks[res.network + 1], apObj) <= 0;
+                okNext = this._networkSortFunction(this._networks[res.network + 1], network) <= 0;
 
             if (!okPrev || !okNext)
                 this._queueCreateSection();
-            else if (apObj.item)
-                apObj.item.updateBestAP(apObj.accessPoints[0]);
+            else if (network.item)
+                network.item.updateBestAP(network.accessPoints[0]);
         }
     },
 
@@ -1284,20 +1180,20 @@ const NMDeviceWireless = new Lang.Class({
 
         let forceupdate = false;
         for (let i = 0; i < this._networks.length; i++) {
-            let apObj = this._networks[i];
-            let connections = apObj.connections;
+            let network = this._networks[i];
+            let connections = network.connections;
             for (let k = 0; k < connections.length; k++) {
                 if (connections[k].get_uuid() == connection.get_uuid()) {
                     // remove the connection from the access point group
-                    connections.splice(k);
+                    connections.splice(k, 1);
                     forceupdate = forceupdate || connections.length == 0;
 
                     if (forceupdate)
                         break;
 
-                    if (apObj.item) {
-                        if (apObj.item instanceof PopupMenu.PopupSubMenuMenuItem) {
-                            let items = apObj.item.menu._getMenuItems();
+                    if (network.item) {
+                        if (network.item instanceof PopupMenu.PopupSubMenuMenuItem) {
+                            let items = network.item.menu._getMenuItems();
                             if (items.length == 2) {
                                 // we need to update the connection list to convert this to a normal item
                                 forceupdate = true;
@@ -1310,8 +1206,8 @@ const NMDeviceWireless = new Lang.Class({
                                 }
                             }
                         } else {
-                            apObj.item.destroy();
-                            apObj.item = null;
+                            network.item.destroy();
+                            network.item = null;
                         }
                     }
                     break;
@@ -1337,13 +1233,13 @@ const NMDeviceWireless = new Lang.Class({
         // find an appropriate access point
         let forceupdate = false;
         for (let i = 0; i < this._networks.length; i++) {
-            let apObj = this._networks[i];
+            let network = this._networks[i];
 
             // Check if connection is valid for any of these access points
-            for (let k = 0; k < apObj.accessPoints.length; k++) {
-                let ap = apObj.accessPoints[k];
+            for (let k = 0; k < network.accessPoints.length; k++) {
+                let ap = network.accessPoints[k];
                 if (ap.connection_valid(connection)) {
-                    apObj.connections.push(connection);
+                    network.connections.push(connection);
                     // this potentially changes the sorting order
                     forceupdate = true;
                     break;
@@ -1364,55 +1260,30 @@ const NMDeviceWireless = new Lang.Class({
         else
             title = _("Connected (private)");
 
-        if (this._activeNetwork)
-            this._activeConnectionItem = new NMNetworkMenuItem(this.device.active_access_point, undefined,
-                                                               { reactive: false });
-        else
-            this._activeConnectionItem = new PopupMenu.PopupImageMenuItem(title,
-                                                                          'network-wireless-connected-symbolic',
-                                                                          { reactive: false });
-        this._activeConnectionItem.setShowDot(true);
+        this._activeConnectionItem = new NMNetworkMenuItem(this.device.active_access_point, undefined,
+                                                           { reactive: false });
+        this._activeConnectionItem.setOrnament(PopupMenu.Ornament.DOT);
     },
 
-    _createAutomaticConnection: function(apObj) {
-        let name;
-        let ssid = NetworkManager.utils_ssid_to_utf8(apObj.ssid);
-        if (ssid) {
-            /* TRANSLATORS: this the automatic wireless connection name (including the network name) */
-            name = _("Auto %s").format(ssid);
-        } else
-            name = _("Auto wireless");
-
-        let connection = new NetworkManager.Connection();
-        connection.add_setting(new NetworkManager.SettingWireless());
-        connection.add_setting(new NetworkManager.SettingConnection({
-            id: name,
-            autoconnect: true, // NetworkManager will know to ignore this if appropriate
-            uuid: NetworkManager.utils_uuid_generate(),
-            type: NetworkManager.SETTING_WIRELESS_SETTING_NAME
-        }));
-        return connection;
-    },
-
-    _createNetworkItem: function(apObj, position) {
-        if(!apObj.accessPoints || apObj.accessPoints.length == 0) {
+    _createNetworkItem: function(network, position) {
+        if(!network.accessPoints || network.accessPoints.length == 0) {
             // this should not happen, but I have no idea why it happens
             return;
         }
 
-        if(apObj.connections.length > 0) {
-            if (apObj.connections.length == 1) {
-                apObj.item = this._createAPItem(apObj.connections[0], apObj, false);
+        if(network.connections.length > 0) {
+            if (network.connections.length == 1) {
+                network.item = this._createAPItem(network.connections[0], network, false);
             } else {
-                let title = apObj.ssidText;
-                apObj.item = new PopupMenu.PopupSubMenuMenuItem(title);
-                for (let i = 0; i < apObj.connections.length; i++)
-                    apObj.item.menu.addMenuItem(this._createAPItem(apObj.connections[i], apObj, true));
+                let title = network.ssidText;
+                network.item = new PopupMenu.PopupSubMenuMenuItem(title);
+                for (let i = 0; i < network.connections.length; i++)
+                    network.item.menu.addMenuItem(this._createAPItem(network.connections[i], network, true));
             }
         } else {
-            apObj.item = new NMNetworkMenuItem(apObj.accessPoints[0]);
-            apObj.item.connect('activate', Lang.bind(this, function() {
-                let accessPoints = apObj.accessPoints;
+            network.item = new NMNetworkMenuItem(network.accessPoints[0]);
+            network.item.connect('activate', Lang.bind(this, function() {
+                let accessPoints = network.accessPoints;
                 if (   (accessPoints[0]._secType == NMAccessPointSecurity.WPA2_ENT)
                     || (accessPoints[0]._secType == NMAccessPointSecurity.WPA_ENT)) {
                     // 802.1x-enabled APs require further configuration, so they're
@@ -1420,23 +1291,23 @@ const NMDeviceWireless = new Lang.Class({
                     Util.spawn(['gnome-control-center', 'network', 'connect-8021x-wifi',
                                 this.device.get_path(), accessPoints[0].dbus_path]);
                 } else {
-                    let connection = this._createAutomaticConnection(apObj);
+                    let connection = new NetworkManager.Connection();
                     this._client.add_and_activate_connection(connection, this.device, accessPoints[0].dbus_path, null)
                 }
             }));
         }
-        apObj.item._apObj = apObj;
+        network.item._network = network;
 
         if (position < NUM_VISIBLE_NETWORKS) {
-            apObj.isMore = false;
-            this.section.addMenuItem(apObj.item, position);
+            network.isMore = false;
+            this.section.addMenuItem(network.item, position);
         } else {
             if (!this._overflowItem) {
                 this._overflowItem = new PopupMenu.PopupSubMenuMenuItem(_("More…"));
                 this.section.addMenuItem(this._overflowItem);
             }
-            this._overflowItem.menu.addMenuItem(apObj.item, position - NUM_VISIBLE_NETWORKS);
-            apObj.isMore = true;
+            this._overflowItem.menu.addMenuItem(network.item, position - NUM_VISIBLE_NETWORKS);
+            network.isMore = true;
         }
     },
 
@@ -1452,13 +1323,13 @@ const NMDeviceWireless = new Lang.Class({
         let activeOffset = this._activeConnectionItem ? 1 : 0;
 
         for(let j = 0; j < this._networks.length; j++) {
-            let apObj = this._networks[j];
-            if (apObj == this._activeNetwork) {
+            let network = this._networks[j];
+            if (network == this._activeNetwork) {
                 activeOffset--;
                 continue;
             }
 
-            this._createNetworkItem(apObj, j + activeOffset);
+            this._createNetworkItem(network, j + activeOffset);
         }
     },
 });
@@ -1680,11 +1551,9 @@ const NMApplet = new Lang.Class({
 
         // Virtual device types
         this._vtypes = { };
-        if (NMGtk) {
-            this._vtypes[NetworkManager.SETTING_VLAN_SETTING_NAME] = NMDeviceVirtual;
-            this._vtypes[NetworkManager.SETTING_BOND_SETTING_NAME] = NMDeviceVirtual;
-            this._vtypes[NetworkManager.SETTING_BRIDGE_SETTING_NAME] = NMDeviceVirtual;
-        }
+        this._vtypes[NetworkManager.SETTING_VLAN_SETTING_NAME] = NMDeviceVirtual;
+        this._vtypes[NetworkManager.SETTING_BOND_SETTING_NAME] = NMDeviceVirtual;
+        this._vtypes[NetworkManager.SETTING_BRIDGE_SETTING_NAME] = NMDeviceVirtual;
 
         // Connection types
         this._ctypes = { };
@@ -1696,11 +1565,9 @@ const NMApplet = new Lang.Class({
         this._ctypes[NetworkManager.SETTING_CDMA_SETTING_NAME] = NMConnectionCategory.WWAN;
         this._ctypes[NetworkManager.SETTING_GSM_SETTING_NAME] = NMConnectionCategory.WWAN;
         this._ctypes[NetworkManager.SETTING_INFINIBAND_SETTING_NAME] = NMConnectionCategory.WIRED;
-        if (NMGtk) {
-            this._ctypes[NetworkManager.SETTING_VLAN_SETTING_NAME] = NMConnectionCategory.VIRTUAL;
-            this._ctypes[NetworkManager.SETTING_BOND_SETTING_NAME] = NMConnectionCategory.VIRTUAL;
-            this._ctypes[NetworkManager.SETTING_BRIDGE_SETTING_NAME] = NMConnectionCategory.VIRTUAL;
-        }
+        this._ctypes[NetworkManager.SETTING_VLAN_SETTING_NAME] = NMConnectionCategory.VIRTUAL;
+        this._ctypes[NetworkManager.SETTING_BOND_SETTING_NAME] = NMConnectionCategory.VIRTUAL;
+        this._ctypes[NetworkManager.SETTING_BRIDGE_SETTING_NAME] = NMConnectionCategory.VIRTUAL;
         this._ctypes[NetworkManager.SETTING_VPN_SETTING_NAME] = NMConnectionCategory.VPN;
 
         NMClient.Client.new_async(null, Lang.bind(this, this._clientGot));
@@ -1768,7 +1635,7 @@ const NMApplet = new Lang.Class({
         this._devices.wireless = {
             section: new PopupMenu.PopupMenuSection(),
             devices: [ ],
-            item: this._makeToggleItem('wireless', _("Wi-Fi"))
+            item: this._makeWirelessToggle()
         };
         this._devices.wireless.section.addMenuItem(this._devices.wireless.item);
         this._devices.wireless.section.actor.hide();
@@ -1815,14 +1682,14 @@ const NMApplet = new Lang.Class({
         }
     },
 
-    _makeToggleItem: function(type, title) {
-        let item = new NMWirelessSectionTitleMenuItem(this._client, type, title);
+    _makeWirelessToggle: function() {
+        let item = new NMWirelessSectionTitleMenuItem(this._client);
         item.connect('enabled-changed', Lang.bind(this, function(item, enabled) {
-            let devices = this._devices[type].devices;
+            let devices = this._devices.wireless.devices;
             devices.forEach(function(dev) {
                 dev.setEnabled(enabled);
             });
-            this._syncSectionTitle(type);
+            this._syncSectionTitle('wireless');
         }));
         return item;
     },
@@ -1900,19 +1767,12 @@ const NMApplet = new Lang.Class({
     },
 
     _syncDeviceNames: function() {
-        if (NMGtk) {
-            let names = NMGtk.utils_disambiguate_device_names(this._nmDevices);
-            for (let i = 0; i < this._nmDevices.length; i++) {
-                let device = this._nmDevices[i];
-                device._description = names[i];
-                if (device._delegate)
-                    device._delegate.syncDescription();
-            }
-        } else {
-            for (let i = 0; i < this._nmDevices.length; i++) {
-                let device = this._nmDevices[i];
+        let names = NMGtk.utils_disambiguate_device_names(this._nmDevices);
+        for (let i = 0; i < this._nmDevices.length; i++) {
+            let device = this._nmDevices[i];
+            device._description = names[i];
+            if (device._delegate)
                 device._delegate.syncDescription();
-            }
         }
     },
 
@@ -1948,11 +1808,6 @@ const NMApplet = new Lang.Class({
         wrapper._deviceStateChangedId = wrapper.connect('state-changed', Lang.bind(this, function(dev) {
             this._syncSectionTitle(dev.category);
         }));
-        wrapper._destroyId = wrapper.connect('destroy', function(wrapper) {
-            wrapper.disconnect(wrapper._activationFailedId);
-            wrapper.disconnect(wrapper._deviceStateChangedId);
-            wrapper.disconnect(wrapper._destroyId);
-        });
 
         let section = this._devices[wrapper.category].section;
         section.addMenuItem(wrapper.statusItem);
@@ -1984,6 +1839,8 @@ const NMApplet = new Lang.Class({
     },
 
     _removeDeviceWrapper: function(wrapper) {
+        wrapper.disconnect(wrapper._activationFailedId);
+        wrapper.disconnect(wrapper._deviceStateChangedId);
         wrapper.destroy();
 
         let devices = this._devices[wrapper.category].devices;
@@ -2047,7 +1904,6 @@ const NMApplet = new Lang.Class({
         let default_ip4 = null;
         let default_ip6 = null;
         let active_vpn = null;
-        let active_any = null;
         for (let i = 0; i < this._activeConnections.length; i++) {
             let a = this._activeConnections[i];
 
@@ -2078,15 +1934,10 @@ const NMApplet = new Lang.Class({
             if (a.default6)
                 default_ip6 = a;
 
-            if (a.state == NetworkManager.ActiveConnectionState.ACTIVATING)
-                activating = a;
-            else if (a.state == NetworkManager.ActiveConnectionState.ACTIVE)
-                active_any = a;
-
-            if (a._type == 'vpn' &&
-                (a.state == NetworkManager.ActiveConnectionState.ACTIVATING ||
-                 a.state == NetworkManager.ActiveConnectionState.ACTIVE))
+            if (a._type == 'vpn')
                 active_vpn = a;
+            else if (a.state == NetworkManager.ActiveConnectionState.ACTIVATING)
+                activating = a;
 
             if (!a._primaryDevice) {
                 if (a._type != NetworkManager.SETTING_VPN_SETTING_NAME) {
@@ -2114,7 +1965,7 @@ const NMApplet = new Lang.Class({
             }
         }
 
-        this._mainConnection = activating || default_ip4 || default_ip6 || active_any || null;
+        this._mainConnection = activating || default_ip4 || default_ip6 || this._activeConnections[0] || null;
         this._vpnConnection = active_vpn;
     },
 
@@ -2140,25 +1991,7 @@ const NMApplet = new Lang.Class({
         return false;
     },
 
-    _readConnections: function() {
-        let connections = this._settings.list_connections();
-        for (let i = 0; i < connections.length; i++) {
-            let connection = connections[i];
-            if (this._ignoreConnection(connection))
-                continue;
-            if (connection._updatedId) {
-                // connection was already seen (for example because NetworkManager was restarted)
-                continue;
-            }
-            connection._removedId = connection.connect('removed', Lang.bind(this, this._connectionRemoved));
-            connection._updatedId = connection.connect('updated', Lang.bind(this, this._updateConnection));
-
-            this._updateConnection(connection);
-            this._connections.push(connection);
-        }
-    },
-
-    _newConnection: function(settings, connection) {
+    _addConnection: function(connection) {
         if (this._ignoreConnection(connection))
             return;
         if (connection._updatedId) {
@@ -2171,14 +2004,22 @@ const NMApplet = new Lang.Class({
 
         this._updateConnection(connection);
         this._connections.push(connection);
+    },
 
+    _readConnections: function() {
+        let connections = this._settings.list_connections();
+        connections.forEach(Lang.bind(this, this._addConnection));
+    },
+
+    _newConnection: function(settings, connection) {
+        this._addConnection(connection);
         this._updateIcon();
     },
 
     _connectionRemoved: function(connection) {
         let pos = this._connections.indexOf(connection);
         if (pos != -1)
-            this._connections.splice(connection);
+            this._connections.splice(connection, 1);
 
         let section = connection._section;
 
