@@ -150,6 +150,14 @@ const ViewSelector = new Lang.Class({
                               Shell.KeyBindingMode.NORMAL |
                               Shell.KeyBindingMode.OVERVIEW,
                               Lang.bind(this, this._toggleAppsPage));
+
+        Main.wm.addKeybinding('toggle-overview',
+                              new Gio.Settings({ schema: SHELL_KEYBINDINGS_SCHEMA }),
+                              Meta.KeyBindingFlags.NONE,
+                              Shell.KeyBindingMode.NORMAL |
+                              Shell.KeyBindingMode.OVERVIEW,
+                              Lang.bind(Main.overview, Main.overview.toggle));
+
     },
 
     _toggleAppsPage: function() {
@@ -178,6 +186,10 @@ const ViewSelector = new Lang.Class({
             Main.overview.fadeInDesktop();
     },
 
+    setWorkspacesFullGeometry: function(geom) {
+        this._workspacesDisplay.setWorkspacesFullGeometry(geom);
+    },
+
     hide: function() {
         this._workspacesDisplay.hide();
     },
