@@ -23,6 +23,18 @@ const MAX_WORKSPACES = 16;
 
 const OVERRIDE_SCHEMA = 'org.gnome.shell.overrides';
 
+function rectEqual(one, two) {
+    if (one == two)
+        return true;
+
+    if (!one || !two)
+        return false;
+
+    return (one.x == two.x &&
+            one.y == two.y &&
+            one.width == two.width &&
+            one.height == two.height);
+}
 
 const WorkspacesView = new Lang.Class({
     Name: 'WorkspacesView',
@@ -43,10 +55,9 @@ const WorkspacesView = new Lang.Class({
                 this._updateWorkspaceActors(false);
             }));
 
-        this._width = 0;
-        this._height = 0;
-        this._x = 0;
-        this._y = 0;
+        this._fullGeometry = null;
+        this._actualGeometry = null;
+
         this._spacing = 0;
         this._animating = false; // tweening
         this._scrolling = false; // swipe-scrolling
@@ -85,8 +96,8 @@ const WorkspacesView = new Lang.Class({
         this._overviewShownId =
             Main.overview.connect('shown',
                                  Lang.bind(this, function() {
-                this.actor.set_clip(this._x, this._y,
-                                    this._width, this._height);
+                this.actor.set_clip(this._fullGeometry.x, this._fullGeometry.y,
+                                    this._fullGeometry.width, this._fullGeometry.height);
         }));
 
         this.scrollAdjustment = new St.Adjustment({ value: activeWorkspaceIndex,
@@ -124,10 +135,8 @@ const WorkspacesView = new Lang.Class({
                 continue;
 
             let ws = new Workspace.Workspace(null, i);
-            ws.setGeometry(monitors[i].x,
-                           monitors[i].y,
-                           monitors[i].width,
-                           monitors[i].height);
+            ws.setFullGeometry(monitors[i]);
+            ws.setActualGeometry(monitors[i]);
             global.overlay_group.add_actor(ws.actor);
             this._extraWorkspaces.push(ws);
         }
@@ -139,18 +148,24 @@ const WorkspacesView = new Lang.Class({
         this._extraWorkspaces = [];
     },
 
-    setGeometry: function(x, y, width, height) {
-      if (this._x == x && this._y == y &&
-          this._width == width && this._height == height)
-          return;
+    setFullGeometry: function(geom) {
+        if (rectEqual(this._fullGeometry, geom))
+            return;
 
-        this._width = width;
-        this._height = height;
-        this._x = x;
-        this._y = y;
+        this._fullGeometry = geom;
 
         for (let i = 0; i < this._workspaces.length; i++)
-            this._workspaces[i].setGeometry(x, y, width, height);
+            this._workspaces[i].setFullGeometry(geom);
+    },
+
+    setActualGeometry: function(geom) {
+        if (rectEqual(this._actualGeometry, geom))
+            return;
+
+        this._actualGeometry = geom;
+
+        for (let i = 0; i < this._workspaces.length; i++)
+            this._workspaces[i].setActualGeometry(geom);
     },
 
     _lookupWorkspaceForMetaWindow: function (metaWindow) {
@@ -210,7 +225,7 @@ const WorkspacesView = new Lang.Class({
 
             Tweener.removeTweens(workspace.actor);
 
-            let y = (w - active) * (this._height + this._spacing);
+            let y = (w - active) * (this._fullGeometry.height + this._spacing);
 
             if (showAnimation) {
                 let params = { y: y,
@@ -281,8 +296,9 @@ const WorkspacesView = new Lang.Class({
 
         if (newNumWorkspaces > oldNumWorkspaces) {
             for (let w = oldNumWorkspaces; w < newNumWorkspaces; w++) {
-                this._workspaces[w].setGeometry(this._x, this._y,
-                                                this._width, this._height);
+                this._workspaces[w].setFullGeometry(this._fullGeometry);
+                if (this._actualGeometry)
+                    this._workspaces[w].setActualGeometry(this._actualGeometry);
                 this.actor.add_actor(this._workspaces[w].actor);
             }
 
@@ -430,7 +446,7 @@ const WorkspacesDisplay = new Lang.Class({
 
     _init: function() {
         this.actor = new St.Widget({ clip_to_allocation: true });
-        this.actor.connect('notify::allocation', Lang.bind(this, this._updateWorkspacesGeometry));
+        this.actor.connect('notify::allocation', Lang.bind(this, this._updateWorkspacesActualGeometry));
         this.actor.connect('parent-set', Lang.bind(this, this._parentSet));
 
         let clickAction = new Clutter.ClickAction()
@@ -484,6 +500,8 @@ const WorkspacesDisplay = new Lang.Class({
 
         this._notifyOpacityId = 0;
         this._scrollEventId = 0;
+
+        this._fullGeometry = null;
     },
 
     _onPan: function(action) {
@@ -572,7 +590,8 @@ const WorkspacesDisplay = new Lang.Class({
             this._workspacesViews.push(view);
         }
 
-        this._updateWorkspacesGeometry();
+        this._updateWorkspacesFullGeometry();
+        this._updateWorkspacesActualGeometry();
 
         for (let i = 0; i < this._workspacesViews.length; i++)
             global.overlay_group.add_actor(this._workspacesViews[i].actor);
@@ -632,31 +651,48 @@ const WorkspacesDisplay = new Lang.Class({
         }));
     },
 
-    _updateWorkspacesGeometry: function() {
+    // This geometry should always be the fullest geometry
+    // the workspaces switcher can ever be allocated, as if
+    // the sliding controls were never slid in at all.
+    setWorkspacesFullGeometry: function(geom) {
+        this._fullGeometry = geom;
+        this._updateWorkspacesFullGeometry();
+    },
+
+    _updateWorkspacesFullGeometry: function() {
         if (!this._workspacesViews.length)
             return;
 
-        let fullWidth = this.actor.allocation.x2 - this.actor.allocation.x1;
-        let fullHeight = this.actor.allocation.y2 - this.actor.allocation.y1;
-
-        let width = fullWidth;
-        let height = fullHeight;
-
-        let [x, y] = this.actor.get_transformed_position();
-
-        let rtl = (Clutter.get_default_text_direction () == Clutter.TextDirection.RTL);
-
         let monitors = Main.layoutManager.monitors;
         let m = 0;
         for (let i = 0; i < monitors.length; i++) {
             if (i == this._primaryIndex) {
-                this._workspacesViews[m].setGeometry(x, y, width, height);
+                this._workspacesViews[m].setFullGeometry(this._fullGeometry);
                 m++;
             } else if (!this._workspacesOnlyOnPrimary) {
-                this._workspacesViews[m].setGeometry(monitors[i].x,
-                                                     monitors[i].y,
-                                                     monitors[i].width,
-                                                     monitors[i].height);
+                this._workspacesViews[m].setFullGeometry(monitors[i]);
+                m++;
+            }
+        }
+    },
+
+    _updateWorkspacesActualGeometry: function() {
+        if (!this._workspacesViews.length)
+            return;
+
+        let [x, y] = this.actor.get_transformed_position();
+        let width = this.actor.allocation.x2 - this.actor.allocation.x1;
+        let height = this.actor.allocation.y2 - this.actor.allocation.y1;
+        let geometry = { x: x, y: y, width: width, height: height };
+
+        let monitors = Main.layoutManager.monitors;
+        let m = 0;
+        for (let i = 0; i < monitors.length; i++) {
+            if (i == this._primaryIndex) {
+                this._workspacesViews[m].setActualGeometry(geometry);
+                m++;
+            } else if (!this._workspacesOnlyOnPrimary) {
+                this._workspacesViews[m].setActualGeometry(monitors[i]);
                 m++;
             }
         }
